/**
 * aliyun.com Inc.
 * Copyright (c) 2004-2017 All Rights Reserved.
 */
package com.alibaba.iot.demo;

import java.io.InputStream;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

import com.alibaba.iot.demo.util.Md5;

/**
 * 
 * @version $Id: SimpleClient.java, v 0.1 2017年1月5日 上午10:34:52 lvjianwen Exp $
 */
public class SimpleClient {
    
    
    public static void main(String...strings)throws Exception {
        String productKey = ""; //这个是设备模版productkey
        String productSecret = "";//这个是设备模版product secret
        final String deviceName = "";//这个是设备名称
        String deviceSecret = "";//这个是设备秘钥 
        String targetServer = "iot-as.aliyuncs.com:80";//接入服务器域名

        //用于测试的topic
        String topic = "/" + productKey + "/" + deviceName + "/update";
        String broker = "ssl://" + targetServer;
       
        //客户端ID格式: productKey : deviceId:idtype:sdkVersion ,如果idtype＝1则 代表设备id是用户自定义的设备名称
        String clientId = productKey + ":" + deviceName + ":" + "1"+":1.0.0";

        MemoryPersistence persistence = new MemoryPersistence();
        SSLSocketFactory socketFactory = createSSLSocket();
        final MqttClient sampleClient = new MqttClient(broker, clientId, persistence);
        MqttConnectOptions connOpts = new MqttConnectOptions();
        connOpts.setMqttVersion(4);// MQTT 3.1.1
        connOpts.setSocketFactory(socketFactory);
        //设置是否自动重连
        connOpts.setAutomaticReconnect(true);

        //用户名签名
        String sign = productKey + productSecret + deviceName + deviceSecret;
        String signUserName = Md5.getInstance().md5_32(sign).toUpperCase();

        connOpts.setUserName(signUserName);
        connOpts.setKeepAliveInterval(80);
        System.out.println(clientId + "，进行连接, 目的地: " + broker);
        sampleClient.connect(connOpts);
        sampleClient.setCallback(new MqttCallback() {
            @Override
            public void connectionLost(Throwable cause) {
                System.out.println("连接失败,原因:" + cause);
            }

            @Override
            public void messageArrived(String topic, MqttMessage message) throws Exception {
                System.out.println(System.currentTimeMillis() +"接收到消息,来至Topic [" + topic + "] , 内容是:["
                                   + new String(message.getPayload(), "UTF-8") + "], num ");
            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken token) {
                //如果是qos 0消息 token.resp是没有回复的
                System.out.println("消息发送成功! " + ((token==null || token.getResponse()==null) ? "null":token.getResponse().getKey()));
            }
        });
        System.out.println(System.currentTimeMillis() + "连接成功:---");
        
        
        //这个是订阅topic 一次订阅永久生效
        sampleClient.subscribe(topic);
        
        /////这里测试发送一条消息
        String content = "Message From Device111:" + deviceName;
        MqttMessage message = new MqttMessage(content.getBytes());
        message.setQos(0);
        System.out.println(System.currentTimeMillis() + "消息发布:---");
        sampleClient.publish(topic, message);
    }


    private static SSLSocketFactory createSSLSocket() throws Exception {
        //CA根证书，可以从官网下载
        InputStream in = SimpleClient.class.getResourceAsStream("/pubkey.crt");
        CertificateFactory cf = CertificateFactory.getInstance("X.509");
        Certificate ca = null;
        try {
            ca = cf.generateCertificate(in);
        } catch (CertificateException e) {
            e.printStackTrace();
        } finally {
            in.close();
        }
        String keyStoreType = KeyStore.getDefaultType();
        KeyStore keyStore = KeyStore.getInstance(keyStoreType);
        keyStore.load(null, null);
        keyStore.setCertificateEntry("ca", ca);
        String tmfAlgorithm = TrustManagerFactory.getDefaultAlgorithm();
        TrustManagerFactory tmf = TrustManagerFactory.getInstance(tmfAlgorithm);
        tmf.init(keyStore);
        SSLContext context = SSLContext.getInstance("TLSV1.2");
        context.init(null, tmf.getTrustManagers(), null);
        SSLSocketFactory socketFactory = context.getSocketFactory();
        return socketFactory;
    }

}
